using System;
using System.Collections.Generic;
using System.Data.SqlClient;
using System.DirectoryServices;
using System.IO;
using System.ServiceProcess;

namespace Microsoft.ConnectionEngine.SampleDeploymentManager
{
    internal static class Utilities
    {
        public static string MapPath(string path)
        {
            return MapPath(AppDomain.CurrentDomain.BaseDirectory, path);
        }

        public static string MapPath(string rootPath, string path)
        {
            //Check that the given path is a relative path. If not then don't change it
            if (!Path.IsPathRooted(path))
            {
                path = Path.Combine(rootPath, path);
            }

            return path;
        }

        public static void XCopy(DirectoryInfo sourceFolder, DirectoryInfo destinationFolder)
        {
            if (!destinationFolder.Exists) destinationFolder.Create();

            FileInfo[] files = sourceFolder.GetFiles();
            foreach (FileInfo file in files)
            {
                file.CopyTo(Path.Combine(destinationFolder.FullName, file.Name));
            }

            DirectoryInfo[] folders = sourceFolder.GetDirectories();
            foreach (DirectoryInfo folder in folders)
            {
                XCopy(folder, new DirectoryInfo(Path.Combine(destinationFolder.FullName, folder.Name)));
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public static string GetCurrentHostName()
        {
            return System.Environment.MachineName;
        }
        
        /// <summary>
        /// 
        /// </summary>
        /// <param name="path"></param>
        /// <param name="parameters"></param>
        public static void SetFileParameters(string path, Dictionary<string, string> parameters)
        {
            string data = ReadDataFromFile(path);

            foreach (KeyValuePair<string, string> parameter in parameters)
            {
                data = data.Replace(parameter.Key, parameter.Value);
            }

            SaveDataToFile(path, data);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        public static string ReadDataFromFile(string path)
        {
            StreamReader reader = new StreamReader(path);

            string data = reader.ReadToEnd();
            reader.Close();

            return data;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="path"></param>
        /// <param name="data"></param>
        public static void SaveDataToFile(string path, string data)
        {
            StreamWriter writer = new StreamWriter(path);

            writer.Write(data);
            writer.Close();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="webServer"></param>
        /// <returns></returns>
        private static bool OSSuppportsApplicationPools(string webServer)
        {
            try
            {
                DirectoryEntry iisApplicationPools = new DirectoryEntry("IIS://" + webServer + "/W3SVC/AppPools");

                foreach (DirectoryEntry de in iisApplicationPools.Children)
                {
                    break;
                }
            }
            catch (System.Runtime.InteropServices.COMException ex)
            {
                if (ex.ErrorCode == -2147024893)
                {
                    return false;
                }
            }

            return true;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="webServer"></param>
        /// <param name="applicationPoolName"></param>
        /// <returns></returns>
        private static DirectoryEntry GetApplicationPool(string webServer, string applicationPoolName)
        {
            if (OSSuppportsApplicationPools(webServer))
            {
                DirectoryEntry iisApplicationPools = new DirectoryEntry("IIS://" + webServer + "/W3SVC/AppPools");

                Dictionary<string, DirectoryEntry> applicationPoolNames = new Dictionary<string, DirectoryEntry>();
                foreach (DirectoryEntry de in iisApplicationPools.Children)
                {
                    applicationPoolNames.Add(de.Name, de);
                }

                if (applicationPoolNames.ContainsKey(applicationPoolName))
                {
                    return applicationPoolNames[applicationPoolName];
                }
            }

            return null;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="webServer"></param>
        /// <param name="applicationPoolName"></param>
        /// <param name="username"></param>
        /// <param name="password"></param>
        public static void CreateIISApplicationPool(string webServer, string applicationPoolName, string username, string password)
        {
            if (OSSuppportsApplicationPools(webServer))
            {
                DirectoryEntry iisApplicationPools = new DirectoryEntry("IIS://" + webServer + "/W3SVC/AppPools");

                Dictionary<string, DirectoryEntry> applicationPoolNames = new Dictionary<string, DirectoryEntry>();
                foreach (DirectoryEntry de in iisApplicationPools.Children)
                {
                    applicationPoolNames.Add(de.Name, de);
                }

                if (GetApplicationPool(webServer, applicationPoolName) == null)
                {
                    DirectoryEntry newApplicationPool = iisApplicationPools.Children.Add(applicationPoolName, "IIsApplicationPool");
                    newApplicationPool.CommitChanges();

                    if (username.Length > 0)
                    {
                        newApplicationPool.Properties["AppPoolIdentityType"][0] = 3;
                        newApplicationPool.Properties["WAMUserName"][0] = username;
                        newApplicationPool.Properties["WAMUserPass"][0] = password;

                        newApplicationPool.CommitChanges();
                    }
                }
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="webServer"></param>
        /// <param name="applicationPoolName"></param>
        public static void RemoveIISApplicationPool(string webServer, string applicationPoolName)
        {
            if (OSSuppportsApplicationPools(webServer))
            {
                DirectoryEntry iisApplicationPools = new DirectoryEntry("IIS://" + webServer + "/W3SVC/AppPools");

                DirectoryEntry applicationPool = GetApplicationPool(webServer, applicationPoolName);

                if (applicationPool != null)
                {
                    try
                    {
                        iisApplicationPools.Children.Remove(applicationPool);
                    }
                    catch
                    {
                    }
                }
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="webServer"></param>
        /// <param name="appName"></param>
        /// <returns></returns>
        public static bool IISVirtualDirExists(string webServer, string appName)
        {
            DirectoryEntry iisAdmin = new System.DirectoryServices.DirectoryEntry("IIS://" + webServer + "/W3SVC/1/Root");
            string className = iisAdmin.SchemaClassName.ToString();

            if ((className.EndsWith("Server")) || (className.EndsWith("VirtualDir")))
            {
                foreach (DirectoryEntry virtualDirectory in iisAdmin.Children)
                {
                    if (string.Compare(virtualDirectory.Name, appName, true) == 0)
                    {
                        return true;
                    }
                }
            }
            else
            {
                throw new Exception("ERROR: Unable to locate the IIS default site or virtual directory node.");
            }

            return false;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="webServer"></param>
        /// <param name="appName"></param>
        /// <param name="path"></param>
        /// <param name="applicationPoolName"></param>
        /// <param name="appPoolUsername"></param>
        /// <param name="appPoolPassword"></param>
        public static void CreateIISVirtualDir(string webServer, string appName, string path, string applicationPoolName, string appPoolUsername, string appPoolPassword)
        {
            DirectoryEntry iisAdmin = new DirectoryEntry("IIS://" + webServer + "/W3SVC/1/Root");
            string className = iisAdmin.SchemaClassName.ToString();

            if ((className.EndsWith("Server")) || (className.EndsWith("VirtualDir")))
            {
                bool canCreateAppPool = OSSuppportsApplicationPools(webServer);
                canCreateAppPool = canCreateAppPool && (applicationPoolName.Length > 0);

                if (canCreateAppPool)
                {
                    CreateIISApplicationPool(webServer, applicationPoolName, appPoolUsername, appPoolPassword);
                }

                RemoveIISVirtualDir(webServer, appName, applicationPoolName);

                DirectoryEntry virtualDirectory = iisAdmin.Children.Add(appName, "IIsWebVirtualDir");

                virtualDirectory.Properties["Path"][0] = path;
                virtualDirectory.Properties["AppFriendlyName"][0] = appName;
                virtualDirectory.Properties["AppRoot"][0] = "/LM/W3SVC/1/Root";
                virtualDirectory.Properties["EnableDirBrowsing"][0] = false;
                virtualDirectory.Properties["AccessRead"][0] = true;
                virtualDirectory.Properties["AccessExecute"][0] = true;
                virtualDirectory.Properties["AccessWrite"][0] = false;
                virtualDirectory.Properties["AccessScript"][0] = true;
                virtualDirectory.Properties["AuthNTLM"][0] = true;
                virtualDirectory.Properties["EnableDefaultDoc"][0] = true;
                virtualDirectory.Properties["DefaultDoc"][0] = "default.htm";
                virtualDirectory.Properties["AspEnableParentPaths"][0] = true;

                if (canCreateAppPool)
                {
                    virtualDirectory.Properties["AppIsolated"][0] = "2";
                }

                virtualDirectory.CommitChanges();

                if ((canCreateAppPool) && (applicationPoolName != null) && (applicationPoolName.Length > 0))
                {
                    object[] createParams = { 0, applicationPoolName, true };

                    virtualDirectory.Invoke("AppCreate3", createParams);
                }
                else
                {
                    virtualDirectory.Invoke("AppCreate", 1);
                }
            }
            else
            {
                throw new Exception("ERROR: Unable to locate the IIS default site or virtual directory node.");
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="webServer"></param>
        /// <param name="appName"></param>
        /// <param name="applicationPoolName"></param>
        public static void RemoveIISVirtualDir(string webServer, string appName, string applicationPoolName)
        {
            DirectoryEntry iisAdmin = new System.DirectoryServices.DirectoryEntry("IIS://" + webServer + "/W3SVC/1/Root");
            string className = iisAdmin.SchemaClassName.ToString();

            if ((className.EndsWith("Server")) || (className.EndsWith("VirtualDir")))
            {
                foreach (DirectoryEntry virtualDirectory in iisAdmin.Children)
                {
                    if (string.Compare(virtualDirectory.Name, appName, true) == 0)
                    {
                        iisAdmin.Invoke("Delete", new string[] { virtualDirectory.SchemaClassName, appName });
                        iisAdmin.CommitChanges();
                        break;
                    }
                }

                bool canRemoveAppPool = OSSuppportsApplicationPools(webServer);
                canRemoveAppPool = canRemoveAppPool && (applicationPoolName.Length > 0);

                if (canRemoveAppPool)
                {
                    RemoveIISApplicationPool(webServer, applicationPoolName);
                }
            }
            else
            {
                throw new Exception("ERROR: Unable to locate the IIS default site or virtual directory node.");
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public static void ReStartIIS()
        {
            StopIIS();

            StartIIS();
        }

        /// <summary>
        /// 
        /// </summary>
        public static void StopIIS()
        {
            ServiceController iis = new ServiceController("IISADMIN");

            if (null != iis)
            {
                do
                {
                    iis.Refresh();
                }
                while (iis.Status == ServiceControllerStatus.ContinuePending ||
                        iis.Status == ServiceControllerStatus.PausePending ||
                        iis.Status == ServiceControllerStatus.StartPending ||
                        iis.Status == ServiceControllerStatus.StopPending);

                if (ServiceControllerStatus.Running == iis.Status ||
                    ServiceControllerStatus.Paused == iis.Status)
                {
                    iis.Stop();
                    iis.WaitForStatus(ServiceControllerStatus.Stopped);
                }

                iis.Close();
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public static void StartIIS()
        {
            ServiceController iis = new ServiceController("W3SVC");

            if (null != iis)
            {
                do
                {
                    iis.Refresh();
                }
                while (iis.Status == ServiceControllerStatus.ContinuePending ||
                        iis.Status == ServiceControllerStatus.PausePending ||
                        iis.Status == ServiceControllerStatus.StartPending ||
                        iis.Status == ServiceControllerStatus.StopPending);

                if (ServiceControllerStatus.Stopped == iis.Status)
                {
                    iis.Start();
                    iis.WaitForStatus(ServiceControllerStatus.Running);
                }
                else if (ServiceControllerStatus.Paused == iis.Status)
                {
                    iis.Continue();
                    iis.WaitForStatus(ServiceControllerStatus.Running);
                }

                iis.Close();
            }
        }

        /// <summary>
        /// Connects to the Master database and run scripts from there.
        /// If scripts must run from a specific database, adds a USE [databasename] as the inital sql command.
        /// All occurrences of [databasename] will be replaced with the databaseName parameter.
        /// </summary>
        /// <param name="serverName">The database server used to create the connection string.</param>
        /// <param name="databaseName">The database name used to create the connection string.</param>
        /// <param name="username">The username used to create the connection string.</param>
        /// <param name="password">The password used to create the connection string.</param>
        /// <param name="sqlFile">A block of transact sql to be executed.</param>
        /// <returns>Returns a boolean indicating the result of the operation.</returns>
        public static bool ExecuteSQLScript(string serverName, string databaseName, string username, string password, string sqlFile)
        {
            return ExecuteSQLScript(serverName, databaseName, username, password, sqlFile, null);
        }

        /// <summary>
        /// Connects to the Master database and run scripts from there.
        /// If scripts must run from a specific database, adds a USE [databasename] as the inital sql command.
        /// All occurrences of [databasename] will be replaced with the databaseName parameter.
        /// </summary>
        /// <param name="serverName">The database server used to create the connection string.</param>
        /// <param name="databaseName">The database name used to create the connection string.</param>
        /// <param name="username">The username used to create the connection string.</param>
        /// <param name="password">The password used to create the connection string.</param>
        /// <param name="sqlFile">A block of transact sql to be executed.</param>
        /// <param name="parameters">Custom parameters</param>
        /// <returns>Returns a boolean indicating the result of the operation.</returns>
        public static bool ExecuteSQLScript(string serverName, string databaseName, string username, string password, string sqlFile, Dictionary<string, string> parameters)
        {
            Boolean result = false;
            SqlConnection connection = null;

            try
            {
                string sql = ReadDataFromFile(sqlFile);
                sql = sql.Replace("[databasename]", databaseName);

                if (parameters != null)
                {
                    foreach (KeyValuePair<string, string> parameter in parameters)
                    {
                        sql = sql.Replace(parameter.Key, parameter.Value);
                    }
                }

                string connectionString = String.Format(@"Data Source={0};Initial Catalog=Master;User Id={1};Password={2};", serverName, username, password);

                connection = new SqlConnection(connectionString);
                SqlCommand cmd = new SqlCommand();
                cmd.CommandType = System.Data.CommandType.Text;
                cmd.Connection = connection;
                connection.Open();

                string[] commands = sql.Split(new string[] { "GO" }, StringSplitOptions.RemoveEmptyEntries);
                for (int x = 0; x < commands.Length; x++)
                {
                    cmd.CommandText = commands[x];
                    cmd.ExecuteNonQuery();
                }

                result = true;
            }
            finally
            {
                if (connection != null)
                {
                    if (connection.State == System.Data.ConnectionState.Open)
                        connection.Close();
                }
            }

            return result;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="serverName"></param>
        /// <param name="databaseName"></param>
        /// <param name="username"></param>
        /// <param name="password"></param>
        /// <param name="serviceProviderID"></param>
        /// <returns></returns>
        public static string GetServiceProviderPublicKey(string serverName, string databaseName, string username, string password, int serviceProviderID)
        {
            string result = "";
            SqlConnection connection = null;

            try
            {
                string connectionString = String.Format(@"Data Source={0};Initial Catalog={1};User Id={2};Password={3};", serverName, databaseName, username, password);

                connection = new SqlConnection(connectionString);
                SqlCommand cmd = new SqlCommand();
                cmd.CommandType = System.Data.CommandType.Text;
                cmd.Connection = connection;
                connection.Open();

                cmd.CommandText = "SELECT [publicKey] FROM [ServiceProvider] WHERE [serviceProviderID] = '" + serviceProviderID.ToString() + "'";
                result = cmd.ExecuteScalar().ToString();
            }
            finally
            {
                if (connection != null)
                {
                    if (connection.State == System.Data.ConnectionState.Open)
                        connection.Close();
                }
            }

            return result;
        }
    }
}
