using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Configuration.Install;
using System.DirectoryServices;
using System.IO;
using System.Windows.Forms;

using common = Microsoft.ConnectionEngine.Common;

namespace ServiceProviderRegisterAdminInstaller
{
    [RunInstaller(true)]
    public partial class Installer : common.Install.Installer
    {
        #region Context Parameter constants
        const string SPRA_WS_SITE = "sprawssite";
        const string SPRA_APP_SITE = "spraappsite";
        #endregion

        #region Other constants
        const string PUBLIC_KEY_FILE_NAME = "SPRAPublicKey.xml";
        const string PRIVATE_KEY_FILE_NAME = "SPRAPrivateKey.xml";
        const string WEB_CONFIG_FILE_NAME = "Web.config";

        const string SPR_PUBLIC_KEY_FILE_NAME = "SPRPublicKey.xml";

        const string DB_DATA_CREATE_FILE_NAME = "SPR-DB-CreateData.sql";

        const string DEFAULT_WEB_SERVICE_APP_NAME = "ServiceProviderRegisterAdminWS";
        const string DEFAULT_WEB_APP_NAME = "ServiceProviderRegisterWebAdmin";
        #endregion

        public Installer()
        {
            InitializeComponent();
        }

        public override void Install(IDictionary savedState)
        {
            FileInfo fileInfo;

            string dbScriptFolder = Path.Combine(InstallFolder, "Database Scripts");

            string serviceInstallFolder = Path.Combine(this.InstallFolder, "WebService");
            string serviceBinFolder = Path.Combine(serviceInstallFolder, "bin");

            string siteInstallFolder = Path.Combine(this.InstallFolder, "WebSite");

            string adminWSSite = this.Context.Parameters[SPRA_WS_SITE];
            if (adminWSSite == null) adminWSSite = DEFAULT_WEB_SERVICE_APP_NAME;
            string adminAppSite = this.Context.Parameters[SPRA_APP_SITE];
            if (adminAppSite == null) adminAppSite = DEFAULT_WEB_APP_NAME;

            #region Fix parameters
            FixParameters();
            #endregion

            #region Create Web Service public and private keys
            common.Crypto.RSAKeyCreator kc = new Microsoft.ConnectionEngine.Common.Crypto.RSAKeyCreator();
            string publicKey = kc.PublicKeyXml;
            string privateKey = kc.PrivateKeyXml;

            fileInfo = new FileInfo(Path.Combine(serviceBinFolder, PUBLIC_KEY_FILE_NAME));
            SaveDataToFile(fileInfo.FullName, publicKey);

            fileInfo = new FileInfo(Path.Combine(serviceBinFolder, PRIVATE_KEY_FILE_NAME));
            SaveDataToFile(fileInfo.FullName, privateKey);
            #endregion

            #region Set Web Service config file parameters
            fileInfo = new FileInfo(Path.Combine(serviceInstallFolder, WEB_CONFIG_FILE_NAME));
            if (fileInfo.Exists)
            {
                Dictionary<string, string> parameters = new Dictionary<string, string>();

                parameters.Add("[installfolder]", serviceInstallFolder);
                parameters.Add("[sprawshost]", GetCurrentHostName());
                foreach (DictionaryEntry parameter in this.Context.Parameters)
                {
                    parameters.Add("[" + parameter.Key.ToString() + "]", parameter.Value.ToString());
                }

                SetFileParameters(fileInfo.FullName, parameters);
            }
            #endregion

            #region Create Web Service IIS Virtual Directory
            string appPoolName = this.Context.Parameters[APP_POOL_NAME];
            string appPoolUsername = this.Context.Parameters[APP_POOL_USER];
            string appPoolPassword = this.Context.Parameters[APP_POOL_PASS];

            CreateIISVirtualDir(GetCurrentHostName(), adminWSSite, serviceInstallFolder, appPoolName, appPoolUsername, appPoolPassword);
            #endregion

            #region Set Web Site config file parameters
            fileInfo = new FileInfo(Path.Combine(siteInstallFolder, WEB_CONFIG_FILE_NAME));
            if (fileInfo.Exists)
            {
                Dictionary<string, string> parameters = new Dictionary<string, string>();

                parameters.Add("[installfolder]", siteInstallFolder);
                parameters.Add("[sprawshost]", GetCurrentHostName());
                foreach (DictionaryEntry parameter in this.Context.Parameters)
                {
                    parameters.Add("[" + parameter.Key.ToString() + "]", parameter.Value.ToString());
                }

                SetFileParameters(fileInfo.FullName, parameters);
            }
            #endregion

            #region Create Web Site IIS Virtual Directory
            CreateIISVirtualDir(GetCurrentHostName(), adminAppSite, siteInstallFolder, appPoolName, appPoolUsername, appPoolPassword);
            #endregion

            #region Setup Service Provider database
            string dbServer = this.Context.Parameters[SPR_DB_HOST];
            string dbName = this.Context.Parameters[SPR_DB_NAME];
            string dbUser = this.Context.Parameters[SPR_DB_USER];
            string dbPassword = this.Context.Parameters[SPR_DB_PASS];

            string sprHost = this.Context.Parameters[SPR_HOST];
            string sprWSSite = this.Context.Parameters[SPR_WS_SITE];

            Dictionary<string, string> dbParameters = new Dictionary<string, string>();

            dbParameters.Add("[sprhost]", sprHost);
            dbParameters.Add("[sprwssite]", sprWSSite);
            dbParameters.Add("[sprapublickey]", publicKey.Replace("'", "''"));
            dbParameters.Add("[sprawssite]", adminWSSite);

            fileInfo = new FileInfo(Path.Combine(dbScriptFolder, DB_DATA_CREATE_FILE_NAME));
            this.ExecuteSQLScript(dbServer, dbName, dbUser, dbPassword, fileInfo.FullName, dbParameters);
            #endregion

            #region Get Service Provider Register public key from database
            string sprPublicKey = GetServiceProviderPublicKey(dbServer, dbName, dbUser, dbPassword, 101);

            fileInfo = new FileInfo(Path.Combine(serviceBinFolder, SPR_PUBLIC_KEY_FILE_NAME));
            SaveDataToFile(fileInfo.FullName, sprPublicKey);
            #endregion

            savedState.Add(SPRA_WS_SITE, adminWSSite);
            savedState.Add(SPRA_APP_SITE, adminAppSite);
            savedState.Add(APP_POOL_NAME, appPoolName);

            base.Install(savedState);
        }

        public override void Uninstall(IDictionary savedState)
        {
            FileInfo fileInfo;

            string serviceInstallFolder = Path.Combine(this.InstallFolder, "WebService");
            string serviceBinFolder = Path.Combine(serviceInstallFolder, "bin");

            string siteInstallFolder = Path.Combine(this.InstallFolder, "WebSite");

            #region Get saved state parameters
            string adminWSSite;
            string adminAppSite;
            string appPoolName;
            if (savedState.Count > 0)
            {
                adminWSSite = savedState[SPRA_WS_SITE].ToString();
                adminAppSite = savedState[SPRA_APP_SITE].ToString();

                appPoolName = savedState[APP_POOL_NAME].ToString();
            }
            else
            {
                adminWSSite = this.Context.Parameters[SPRA_WS_SITE];
                if (adminWSSite == null) adminWSSite = DEFAULT_WEB_SERVICE_APP_NAME;
                adminAppSite = this.Context.Parameters[SPRA_APP_SITE];
                if (adminAppSite == null) adminAppSite = DEFAULT_WEB_APP_NAME;

                appPoolName = this.Context.Parameters[APP_POOL_NAME];
                if (appPoolName == null) appPoolName = DEFAULT_APP_POOL_NAME;
            }
            #endregion

            ReStartIIS();

            #region Remove Service Provider Register public key
            fileInfo = new FileInfo(Path.Combine(serviceBinFolder, SPR_PUBLIC_KEY_FILE_NAME));
            if (fileInfo.Exists) fileInfo.Delete();
            #endregion

            #region Remove Web Site IIS Virtual Directory
            RemoveIISVirtualDir(GetCurrentHostName(), adminAppSite, appPoolName);
            #endregion

            #region Remove Web Service IIS Virtual Directory
            RemoveIISVirtualDir(GetCurrentHostName(), adminWSSite, appPoolName);
            #endregion

            #region Remove Web Service public and private keys
            fileInfo = new FileInfo(Path.Combine(serviceBinFolder, PUBLIC_KEY_FILE_NAME));
            if (fileInfo.Exists) fileInfo.Delete();

            fileInfo = new FileInfo(Path.Combine(serviceBinFolder, PRIVATE_KEY_FILE_NAME));
            if (fileInfo.Exists) fileInfo.Delete();
            #endregion

            base.Uninstall(savedState);
        }

        public override void Rollback(IDictionary savedState)
        {
            FileInfo fileInfo;

            string serviceInstallFolder = Path.Combine(this.InstallFolder, "WebService");
            string serviceBinFolder = Path.Combine(serviceInstallFolder, "bin");

            string siteInstallFolder = Path.Combine(this.InstallFolder, "WebSite");

            #region Get saved state parameters
            string adminWSSite = this.Context.Parameters[SPRA_WS_SITE];
            if (adminWSSite == null) adminWSSite = DEFAULT_WEB_SERVICE_APP_NAME;
            string adminAppSite = this.Context.Parameters[SPRA_APP_SITE];
            if (adminAppSite == null) adminAppSite = DEFAULT_WEB_APP_NAME;
            #endregion

            #region Fix parameters
            FixParameters();
            #endregion

            ReStartIIS();

            #region Remove Service Provider Register public key
            fileInfo = new FileInfo(Path.Combine(serviceBinFolder, SPR_PUBLIC_KEY_FILE_NAME));
            if (fileInfo.Exists) fileInfo.Delete();
            #endregion

            #region Remove Web Site IIS Virtual Directory
            string appPoolName = this.Context.Parameters[APP_POOL_NAME];

            RemoveIISVirtualDir(GetCurrentHostName(), adminAppSite, appPoolName);
            #endregion

            #region Remove Web Service IIS Virtual Directory
            RemoveIISVirtualDir(GetCurrentHostName(), adminWSSite, appPoolName);
            #endregion

            #region Remove Web Service public and private keys
            fileInfo = new FileInfo(Path.Combine(serviceBinFolder, PUBLIC_KEY_FILE_NAME));
            if (fileInfo.Exists) fileInfo.Delete();

            fileInfo = new FileInfo(Path.Combine(serviceBinFolder, PRIVATE_KEY_FILE_NAME));
            if (fileInfo.Exists) fileInfo.Delete();
            #endregion

            if (savedState.Count > 0)
            {
                base.Rollback(savedState);
            }
        }

        #region Protected methods
        //protected override void FixParameters()
        //{
        //    string localhostName = this.GetCurrentHostName();
        //    string value;

        //    value = this.Context.Parameters[SPR_DB_HOST];
        //    if (value != null)
        //    {
        //        if ((string.Compare(value, ".") == 0) ||
        //            (string.Compare(value, "localhost", true) == 0))
        //        {
        //            this.Context.Parameters[SPR_DB_HOST] = localhostName;
        //        }
        //    }

        //    base.FixParameters();
        //}
        #endregion

        #region Private methods
        private void ShowParameters()
        {
            MessageBox.Show(GetParamtersForDisplay(), "Setup Parameters");
        }

        private void ShowSavedState(IDictionary savedState)
        {
            string output = "";

            foreach (DictionaryEntry item in savedState)
            {
                if (output.Length > 0) output += "\n";
                output += "[" + item.Key.ToString() + "] = '" + item.Value.ToString() + "'";
            }

            MessageBox.Show(GetSavedStateForDisplay(savedState), "Setup Saved State");
        }
        #endregion
    }
}
